var StatusmenuAssistant = Class.create({
	initialize: function(barAssistant) {
		this.barAssistant = barAssistant;

		this.isVisible = true;
		this.isEditing = false;

		this.prevDrawer = null;

		this.menuItems = new Array();
		this.editItems = new Array();

		this.quickAction = "Triggers Locking";
		this.hiddenItems = new Array();
	},
	
	setup: function() {
		this.modelScroller = {'scrollbars': false, 'mode': "vertical"};
		
		this.controller.setupWidget('scroller', {}, this.modelScroller);
		
		this.menuScroller = this.controller.get('scroller');

		// Set up the menu content

		this.controller.get('menu_header').innerHTML = $L("Status: Default Settings");

		this.controller.get('menu_action').innerHTML = $L('Configure Mode Switcher');
		
		this.menuItemsModel = {'items': this.menuItems};

		this.controller.setupWidget('menu_items', 
			{itemTemplate:'statusmenu/listitem'}, 
			this.menuItemsModel);

		this.modelEditItems = {'items': this.editItems};

		this.controller.setupWidget('edit_items', 
			{itemTemplate:'statusmenu/listitem-edit'}, 
			this.modelEditItems);
		
		this.controller.setupWidget('modedrawer', {});

		// Set up the menu functionality
		
		this.scrim = this.controller.sceneElement.querySelector('div[x-mojo-scrim]');

		this.scrim.style.opacity = 0;
				
		this.systemMenu = this.controller.sceneElement.querySelector('div[x-mojo-dialog]');
		
		this.systemMenuTop = this.systemMenu.offsetTop;
		
		this.systemMenu.style.top = (-1 * this.systemMenu.offsetHeight) + 'px';					

		this.controller.listen(this.controller.document, Mojo.Event.deactivate, this.close.bindAsEventListener(this));
		
		// Load preferences for the menu

		this.getPreferencesRequest = new Mojo.Service.Request('palm://com.palm.systemservice/', {
			method: 'getPreferences', parameters: {subscribe: false, keys: ['statusMenuCfg']},
			onSuccess: this.handlePreferences.bind(this) });
	},

	close: function() {
		if(this.isVisible)
			this.toggleStatusMenu();
		
		this.menuScroller.mojo.revealTop(0);
	},
		
	cleanup: function() {
		Mojo.Log.error("Status Menu - Clean up called");				
	},
		
	activate: function() {
		this.controller.get('menu_action').addEventListener(Mojo.Event.tap, this.handleActionTap.bindAsEventListener(this));

		this.controller.get('menu_items').addEventListener(Mojo.Event.listTap, this.handleItemTap.bindAsEventListener(this));
		this.controller.get('edit_items').addEventListener(Mojo.Event.listTap, this.handleItemTap.bindAsEventListener(this));
	
		this.controller.get('scrim').addEventListener(Mojo.Event.tap, this.toggleStatusMenu.bindAsEventListener(this));

		//Close the menu because this is a first launch
		
		this.toggleStatusMenu();						
	},	

	savePreferences: function() {
		this.hiddenItems.clear();
	
		for(var i = 0; i < this.editItems.length ; i++) {
			if(this.editItems[i].label == "hide")
				this.hiddenItems.push(this.editItems[i].title);
		}
		
		if(this.setPreferencesRequest)
			this.setPreferencesRequest.cancel();
	
		this.setPreferencesRequest = new Mojo.Service.Request('palm://com.palm.systemservice/', {
			method: 'setPreferences', parameters: {'statusMenuCfg': {hidden: this.hiddenItems, action: this.quickAction}} });
	},

	handlePreferences: function(payload) {
		if(payload != undefined) {
			if(payload.statusMenuCfg != undefined) {
				this.hiddenItems = payload.statusMenuCfg.hidden;
				this.quickAction = payload.statusMenuCfg.action;
			}
		}
	},
	
	handleActionTap: function(event) {
		var config = this.barAssistant.getModeSwitcherConfig();

		if(this.isEditing) {
			if(config.modesConfig) {
				if((event.up) && ( event.up.altKey || event.up.metaKey))
					config.modesConfig.reverse();

				if(this.quickAction == "Triggers Locking") {
					if(config.modesConfig.length > 0)
						this.quickAction = config.modesConfig[0].name;
				}
				else {
					var curAction = this.quickAction;
			
					this.quickAction = "Triggers Locking";			
					
					for(var i = 0; i < config.modesConfig.length; i++) {
						if(config.modesConfig[i].name == curAction) {
							if((i+1) < config.modesConfig.length)
								this.quickAction = config.modesConfig[i+1].name;
						
							break;
						}
					}
				}

				if((event.up) && ( event.up.altKey || event.up.metaKey))
					config.modesConfig.reverse();
			}
									
			this.controller.get('menu_action').innerHTML = $L("Toggle " + this.quickAction);

			this.savePreferences();
		}
		else {
			var appControl = Mojo.Controller.getAppController();
	
			if((!config.modeSwitcher) || (config.modeSwitcher.activated == 0))
				appControl.launch("com.palm.org.e-lnx.wee.apps.modeswitcher", null, null, null);
			else if(this.quickAction == "Triggers Locking") {
				if(config.modeSwitcher.modeLocked == "yes") {
					appControl.launch("com.palm.org.e-lnx.wee.apps.modeswitcher", {"action": "control", "event": "unlock"}, null, null);
					appControl.launch("com.palm.org.e-lnx.wee.apps.modeswitcher", {"action": "trigger", "event": "manual", "data": "All Modes"}, null, null);
				}
				else if(config.modeSwitcher.modeLocked == "no")
					appControl.launch("com.palm.org.e-lnx.wee.apps.modeswitcher", {"action": "control", "event": "lock"}, null, null);
			}
			else {
				appControl.launch("com.palm.org.e-lnx.wee.apps.modeswitcher", {"action": "execute", "event": "toggle", "name": this.quickAction}, null, null);
			}
			
			this.toggleStatusMenu.bind(this).delay(0.2);
		}
	},
	
	handleItemTap: function(event) {
		if(this.isEditing) {
			if(this.modelEditItems.items[event.index].label == "hide")
				this.modelEditItems.items[event.index].label = "show";
			else
				this.modelEditItems.items[event.index].label = "hide";
		
			this.controller.modelChanged(this.modelEditItems, this);
			
			this.savePreferences();
		}
		else {
			var appControl = Mojo.Controller.getAppController();

			if(event.originalEvent.target.id) {
				if(event.originalEvent.target.id == "mode_action") {
					appControl.launch("com.palm.org.e-lnx.wee.apps.modeswitcher", {"action": "execute", "event": "toggle", "name": event.item.title}, null, null);
				}
				else if(event.originalEvent.target.id == "mode_config") {
					appControl.launch("com.palm.org.e-lnx.wee.apps.modeswitcher", {"action": "config", "event": "edit", "name": event.item.title}, null, null);
				}

				this.toggleStatusMenu();
			}
			else {
				if((event.up) && ( event.up.altKey || event.up.metaKey)) {
					appControl.launch("com.palm.org.e-lnx.wee.apps.modeswitcher", {"action": "execute", "event": "toggle", "name": event.item.title}, null, null);
				}
				else {
					var drawers = this.controller.document.getElementsByName('mode_drawer');

					if((this.prevDrawer != null) && (this.prevDrawer != event.index)) {
						drawers[this.prevDrawer].mojo.setOpenState(false);
					}
	
					this.prevDrawer = event.index;

					drawers[event.index].mojo.toggleState();
				}
			}
		}
	},

	updateMenuItems: function() {
		this.menuItems.clear();

		var config = this.barAssistant.getModeSwitcherConfig();
		
		if((!config.modeSwitcher) || (config.modeSwitcher.activated == 0))
			this.controller.get('menu_header').innerHTML = $L("Mode Switcher Disabled");
		else if(config.modifierModes.length == 0) {
			if(config.currentMode.type == "default")
				this.controller.get('menu_header').innerHTML = $L("Status: Default Settings");
			else
				this.controller.get('menu_header').innerHTML = $L("Status: Normal Settings");
		}
		else
			this.controller.get('menu_header').innerHTML = $L("Status: Modified Settings");

		if((!config.modeSwitcher) || (config.modeSwitcher.activated == 0))
			this.controller.get('menu_action').innerHTML = $L('Configure Mode Switcher');
		else if(this.quickAction == "Triggers Locking") {
			if(config.modeSwitcher.modeLocked == "yes")
				this.controller.get('menu_action').innerHTML = $L('Enable Mode Triggers');
			else if(config.modeSwitcher.modeLocked == "no")
				this.controller.get('menu_action').innerHTML = $L('Disable Mode Triggers');				
		}
		else {
			var exists = false;
		
			for(var i = 0; i < config.modesConfig.length; i++) {
				if(this.quickAction == config.modesConfig[i].name) {
					exists = true;
				
					if(config.modesConfig[i].type == "normal") {
						if(this.quickAction == config.currentMode.name)
							this.controller.get('menu_action').innerHTML = $L("Close") + " " + this.quickAction + " " + $L("Now");
						else
							this.controller.get('menu_action').innerHTML = $L("Switch to") + " " + this.quickAction;
					}
					else if(config.modesConfig[i].type == "modifier") {
						if(config.modifierModes.indexOf(config.modesConfig[i].name) != -1)
							this.controller.get('menu_action').innerHTML = $L("Close") + " " + this.quickAction + " " + $L("Now");
						else
							this.controller.get('menu_action').innerHTML = $L("Start") + " " + this.quickAction + $L("Now");	
					}
				
					break;
				}
			}
			
			if(!exists) {
				this.quickAction = "Triggers Locking";
				
				if(config.modeSwitcher.modeLocked == "yes")
					this.controller.get('menu_action').innerHTML = $L("Enable Mode Triggers");
				else if(config.modeSwitcher.modeLocked == "no")
					this.controller.get('menu_action').innerHTML = $L("Disable Mode Triggers");				
			}
		}

		if(config.modesConfig) {
			for(var i = 0; i < config.modesConfig.length; i++) {
				if(this.hiddenItems.indexOf(config.modesConfig[i].name) == -1) {
					if(config.modesConfig[i].type == "normal") {
						if(config.currentMode.name == config.modesConfig[i].name)
							this.menuItems.push({title: config.modesConfig[i].name, label: "ON", action: "Close This Mode Now", open: false});
						else
							this.menuItems.push({title: config.modesConfig[i].name, label: "OFF", action: "Switch to This Mode", open: false});
					}
					else if(config.modesConfig[i].type == "modifier") {
						if(config.modifierModes.indexOf(config.modesConfig[i].name) != -1)
							this.menuItems.push({title: config.modesConfig[i].name, label: "ON", action: "Close This Mode Now", open: false});
						else
							this.menuItems.push({title: config.modesConfig[i].name, label: "OFF", action: "Start This Mode Now", open: false});
					}
				}			
			}
		}
				
		this.controller.modelChanged(this.menuItemsModel, this);			
	},

	updateEditItems: function() {
		this.editItems.clear();

		var config = this.barAssistant.getModeSwitcherConfig();

		this.controller.get('menu_header').innerHTML = $L("Menu Content Settings");

		var exists = false;

		if(config.modesConfig) {
			for(var i = 0; i< config.modesConfig.length; i++) {
				if(config.modesConfig[i].name == this.quickAction)
					exists = true;
		
				if(this.hiddenItems.indexOf(config.modesConfig[i].name) == -1)
					this.editItems.push({title: config.modesConfig[i].name, label: "show"});
				else
					this.editItems.push({title: config.modesConfig[i].name, label: "hide"});
			}
		}
				
		if(!exists) {
			this.quickAction = "Triggers Locking";
			
			this.savePreferences();
		}

		this.controller.get('menu_action').innerHTML = $L("Toggle") + " " + $L(this.quickAction);

		this.controller.modelChanged(this.modelEditItems, this);			
	},

	toggleStatusMenu: function(edit) {
		if(this.isVisible) {			
			this.isVisible = false;
			this.isEditing = false;

			var animateScrim = function(){
				Mojo.Animation.animateStyle(this.scrim, 'opacity', 'bezier', {
					from: 1,
					to: 0,
					duration: .15,
					curve:'over-easy',
					reverse:true,
					styleSetter: function(element,value){
						element.style.opacity = value;
						}.curry(this.scrim).bind(this),
					onComplete: this.controller.stageController.deactivate.bind(this.controller.stageController)
				});
			}

			Mojo.Animation.animateStyle(this.systemMenu, 'top', 'bezier', {
				from: this.systemMenuTop,
				to: (-1 * this.systemMenu.offsetHeight),
				duration: .15,
				curve:'ease-out',
				onComplete:animateScrim.bind(this)
			});
		}
		else {
			this.isVisible = true;
			
			if(edit) {
				this.isEditing = true;
			
				this.controller.get('menu_items').hide();
				this.controller.get('edit_items').show();

				this.updateEditItems();			
			}
			else {
				this.isEditing = false;
				
				this.controller.get('menu_items').show();
				this.controller.get('edit_items').hide();

				this.updateMenuItems();
			}
								
			this.controller.stageController.activate();
			
			var animateDialog = function(){
				Mojo.Animation.animateStyle(this.systemMenu, 'top', 'bezier', {
					from: (-1 * this.systemMenu.offsetHeight),
					to: this.systemMenuTop,
					duration: .15,
					curve:'ease-in'
				});
			}
            
			Mojo.Animation.animateStyle(this.scrim, 'opacity', 'bezier', {
				from: 0,
				to: 1,
				duration: .15,
				curve:'over-easy',
				reverse:false,
				styleSetter: function(element,value){
					element.style.opacity = value;
				}.curry(this.scrim).bind(this),
				onComplete: animateDialog.bind(this)
			});
		}
	},
});

