var TodaymenuAssistant = Class.create({
	initialize: function(barAssistant) {
		this.barAssistant = barAssistant;

		this.isVisible = true;
		this.isEditing = false;
		
		this.startDay = 0;
		
		this.eventDay = 0;
		this.eventDays = 3;

		this.pastEvents = {};
		
		this.menuItems = new Array();
		this.editItems = new Array();

		this.hiddenItems = new Array();
	},
	
	setup: function() {
		var date = new Date();
		var week = this.getWeek(date);
		
		this.pastEvents = {'date': date, 'list': []};

		this.modelScroller = {'scrollbars': false, 'mode': "vertical"};
		
		this.controller.setupWidget('scroller', {}, this.modelScroller);
		
		this.menuScroller = this.controller.get('scroller');
		
		// Set up the menu content
		
		this.controller.get('menu_header').innerHTML = Mojo.Format.formatDate(date, {'date': "EEE "}) + 
			Mojo.Format.formatDate(date, {'date': "medium"}) + ", Week " + week;

		this.controller.get('menu_action').innerHTML = $L("Show Tomorrow's Events");
		
		this.modelMenuItems = {'items': this.menuItems};
		
		this.controller.setupWidget('menu_items', 
			{itemTemplate:'todaymenu/listitem'}, 
			this.modelMenuItems);				

		this.modelEditItems = {'items': this.editItems};

		this.controller.setupWidget('edit_items', 
			{itemTemplate:'todaymenu/listitem-edit'}, 
			this.modelEditItems);

		// Set up the menu functionality
		
		this.scrim = this.controller.sceneElement.querySelector('div[x-mojo-scrim]');

		this.scrim.style.opacity = 0;
		
		this.systemMenu = this.controller.sceneElement.querySelector('div[x-mojo-dialog]');

		this.systemMenuTop = this.systemMenu.offsetTop;

		this.systemMenu.style.top = (-1 * this.systemMenu.offsetHeight) + 'px';		

		this.controller.listen(this.controller.document, Mojo.Event.deactivate, this.close.bindAsEventListener(this));
		
		// Load preferences for the menu
		
		this.getPreferencesRequest = new Mojo.Service.Request('palm://com.palm.systemservice/', {
			method: 'getPreferences', parameters: {subscribe: false, keys: ['todayMenuCfg']},
			onSuccess: this.handlePreferences.bind(this) });			
	},

	close: function() {
		if(this.isVisible)
			this.toggleTodayMenu();
		
		this.menuScroller.mojo.revealTop(0);
	},
	
	cleanup: function() {
		Mojo.Log.error("Today Menu - Clean up called");				
	},	
	
	activate: function() {
		this.controller.get('menu_header').addEventListener(Mojo.Event.tap, this.handleHeaderTap.bindAsEventListener(this));
		this.controller.get('menu_action').addEventListener(Mojo.Event.tap, this.handleActionTap.bindAsEventListener(this));
		
		this.controller.get('menu_items').addEventListener(Mojo.Event.listTap, this.handleItemTap.bindAsEventListener(this));
		this.controller.get('edit_items').addEventListener(Mojo.Event.listTap, this.handleItemTap.bindAsEventListener(this));
		
		this.controller.get('scrim').addEventListener(Mojo.Event.tap, this.toggleTodayMenu.bindAsEventListener(this));
		
		// Close the menu because this is a first launch
		
		this.toggleTodayMenu();		
	},	

	savePreferences: function() {
		this.hiddenItems.clear();
	
		for(var i = 0; i < this.editItems.length ; i++) {
			if(this.editItems[i].label == "hide")
				this.hiddenItems.push("id" + this.editItems[i].id);
		}
		
		if(this.setPreferencesRequest)
			this.setPreferencesRequest.cancel();

		this.setPreferencesRequest = new Mojo.Service.Request('palm://com.palm.systemservice/', {
			method: 'setPreferences', parameters: {'todayMenuCfg': {hidden: this.hiddenItems, days: this.eventDays}} });
	},
	
	handlePreferences: function(payload) {
		if((payload != undefined) && (payload.todayMenuCfg != undefined)) {
			this.hiddenItems = payload.todayMenuCfg.hidden;
			this.eventDays = payload.todayMenuCfg.days;
		}
	},
	
	handleHeaderTap: function(event) {
		if(!this.isEditing) {
			if((event.up) && ( event.up.altKey || event.up.metaKey)) {	
				this.pastEvents.list.clear();
	
				this.updateMenuItems();
			}
			else {
				this.toggleTodayMenu();

				var date = new Date();
				
				date.setDate(date.getDate() + this.eventDay);
		
				var time = "" + date.getTime();
		
				var appController = Mojo.Controller.getAppController();
			
				appController.launch("com.palm.app.calendar", {'date': time}, null, null);
			}
		}
	},
	
	handleActionTap: function(event) {
		if(this.isEditing) {
			if(this.eventDays == 3)
				this.eventDays = 7;
			else
				this.eventDays = 3;
			
			this.controller.get('menu_action').innerHTML = $L('Show Events for ' + this.eventDays + ' Days');
				
			this.savePreferences();
		}
		else {	
			if((event.up) && ( event.up.altKey || event.up.metaKey))
				this.eventDay--;
			else
				this.eventDay++;
		
			if(this.eventDay == -1)
				this.eventDay = this.eventDays - 1;
		
			if(this.eventDay == this.eventDays)
				this.eventDay = 0;
			
			this.updateMenuItems();
		}
	},
	
	handleItemTap: function(event) {
		if(this.isEditing) {
			if((event.originalEvent.up) && ( event.originalEvent.up.altKey || event.originalEvent.up.metaKey)) {
				if(this.modelEditItems.items[event.index].title == this.modelEditItems.items[event.index].name)
					this.modelEditItems.items[event.index].title = this.modelEditItems.items[event.index].source;
				else
					this.modelEditItems.items[event.index].title = this.modelEditItems.items[event.index].name;
			}
			else {
				if(this.modelEditItems.items[event.index].label == "hide")
					this.modelEditItems.items[event.index].label = "show";
				else
					this.modelEditItems.items[event.index].label = "hide";
					
				this.savePreferences();
			}
		
			this.controller.modelChanged(this.modelEditItems, this);
		}
		else {
			if((event.originalEvent.up) && ( event.originalEvent.up.altKey || event.originalEvent.up.metaKey)) {
				if(this.modelMenuItems.items[event.index].label == this.modelMenuItems.items[event.index].stime)
					this.modelMenuItems.items[event.index].label = this.modelMenuItems.items[event.index].etime;
				else
					this.modelMenuItems.items[event.index].label = this.modelMenuItems.items[event.index].stime;
				
				this.controller.modelChanged(this.modelMenuItems, this);
			}
			else {
				this.toggleTodayMenu();
			
				var appController = Mojo.Controller.getAppController();
			
				appController.launch("com.palm.app.calendar", {'details': this.modelMenuItems.items[event.index].id}, null, null);
			}
		}
	},

	updateMenuItems: function() {
		this.menuItems.clear();

		var curDate = new Date();

		var date = new Date(curDate.getTime() + this.eventDay * 86400000);

		var week = this.getWeek(date);
					
		this.controller.get('menu_header').innerHTML = Mojo.Format.formatDate(date, {'date': 'EEE '}) + 
			Mojo.Format.formatDate(date,{'date': 'medium'}) + ", Week " + week;

		if(this.eventDay == this.eventDays - 1)
			this.controller.get('menu_action').innerHTML = $L("Show Today's Events");
		else
			this.controller.get('menu_action').innerHTML = $L("Show Tomorrow's Events");
		
		this.controller.serviceRequest('palm://com.palm.calendar/', { method: 'getCalendarPrefs', parameters: {},
			onSuccess: function(response) {
				this.startDay = response.startOfWeek;
			
				this.controller.serviceRequest('palm://com.palm.calendar/', { method: 'getEvents',
					parameters: {calendarId: 'all', startDate: date.getTime(), endDate: date.getTime(), subscribe: true},
					onSuccess: function(response) { 
						var adevents = new Array();
						var wevents = new Array();
						var events = new Array();
						
						var ignore = new RegExp("/*iGnore-ME*");
						
						response.days.each(function(day) {
							day.allDayEvents.each(function(allday) {
								if(this.hiddenItems.indexOf(allday.calendarId) == -1) {
									if(allday.subject.substring(0, 12) == "Forecast for") {
										var evName = allday.subject.replace(/ \(.*?\)/, "");
										var evSTime = allday.subject.replace(/.*?\(/, "").replace(/ \|.*?\)/, "");
										var evETime = allday.subject.replace(/.*?\| /, "").replace(/\)/, "");
																		
										wevents.push({id: allday.id, title: evName, label: evSTime, stime: evSTime, etime: evETime});
									}
									else if(allday.subject.substring(0, 13) == "First quarter") {
										var evName = "Moon - First Quarter";
										var evSTime = allday.subject.substring(14);
										var evETime = allday.subject.substring(14);
																		
										wevents.push({id: allday.id, title: evName, label: evSTime, stime: evSTime, etime: evETime});
									}
									else if(allday.subject.substring(0, 12) == "Last quarter") {
										var evName = "Moon - Last Quarter";
										var evSTime = allday.subject.substring(13);
										var evETime = allday.subject.substring(13);
																		
										wevents.push({id: allday.id, title: evName, label: evSTime, stime: evSTime, etime: evETime});
									}
									else if(allday.subject.substring(0, 8) == "New moon") {
										var evName = "Moon - New Moon";
										var evSTime = allday.subject.substring(9);
										var evETime = allday.subject.substring(9);
																		
										wevents.push({id: allday.id, title: evName, label: evSTime, stime: evSTime, etime: evETime});
									}
									else if(allday.subject.substring(0, 9) == "Full moon") {
										var evName = "Moon - Full Moon";
										var evSTime = allday.subject.substring(10);
										var evETime = allday.subject.substring(10);
																		
										wevents.push({id: allday.id, title: evName, label: evSTime, stime: evSTime, etime: evETime});
									}
									else if((allday.subject.match(ignore) == null) && (allday.note.match(ignore) == null)) {
										adevents.push({id: allday.id, title: allday.subject, label: "DAY", stime: "DAY", etime: "DAY"});
									}
								}
							}, this);

							day.events.each(function(event) {
								if(this.hiddenItems.indexOf(event.calendarId) == -1) {
									var start = new Date(event.start);
									var end = new Date(event.end);
									
									var evSTime = Mojo.Format.formatDate(start, {time: "short"});
									var evETime = Mojo.Format.formatDate(end, {time: "short"});
								
									if(this.pastEvents.date.getDate() != curDate.getDate()) {
										this.pastEvents.date = curDate;
										this.pastEvents.list.clear();
									}
								
									if((event.subject.match(ignore) == null) && (event.note.match(ignore) == null)) {
										if(this.pastEvents.list.indexOf(event.id) == -1) {
											if(curDate > end)
												this.pastEvents.list.push(event.id);
											
											events.push({id: event.id, title: event.subject, label: evSTime, stime: evSTime, etime: evETime, date: start});
										}
									}
								}
							}, this);
						}, this);

						adevents.sort(this.sortAlldayEvents);		
						wevents.sort(this.sortAlldayEvents);
						events.sort(this.sortNormalEvents);

						for(var i = 0; i < wevents.length; i++)
							this.menuItems.push(wevents[i]);

						for(var i = 0; i < adevents.length; i++)
							this.menuItems.push(adevents[i]);

						for(var i = 0; i < events.length; i++)
							this.menuItems.push(events[i]);

						this.controller.modelChanged(this.modelMenuItems, this);
					}.bind(this) });
			}.bind(this) });
	},
	
	updateEditItems: function() {
		this.editItems.clear();
	
		this.controller.get('menu_header').innerHTML = $L("Menu Content Settings");
		
		this.controller.get('menu_action').innerHTML = $L("Show Events for") + " " + this.eventDays + " " + $L("Days");
	
		this.controller.serviceRequest('palm://com.palm.calendar/', {
			method: 'getCalendarsByAccount', parameters: {subscribe: false},
			onSuccess: function(response) {
				for(var i = 0; i < response.accounts.length; i++) {
					for(var j = 0; j < response.accounts[i].calendars.length; j++) {
						if(this.hiddenItems.indexOf("id" + response.accounts[i].calendars[j].id) == -1)
							this.editItems.push({title: response.accounts[i].calendars[j].name, name: response.accounts[i].calendars[j].name, source: response.accounts[i].calendars[j].syncSource, id: response.accounts[i].calendars[j].id, label: "show"});
						else
							this.editItems.push({title: response.accounts[i].calendars[j].name, name: response.accounts[i].calendars[j].name, source: response.accounts[i].calendars[j].syncSource, id: response.accounts[i].calendars[j].id, label: "hide"});
					}
				}

				this.controller.modelChanged(this.modelEditItems, this);			
			}.bind(this) });
	},
		
	toggleTodayMenu: function(edit) {
		if(this.isVisible) {			
			this.isVisible = false;

			var animateScrim = function(){
				Mojo.Animation.animateStyle(this.scrim, 'opacity', 'bezier', {
					from: 1,
					to: 0,
					duration: .15,
					curve:'over-easy',
					reverse:true,
					styleSetter: function(element,value){
						element.style.opacity = value;
					}.curry(this.scrim).bind(this),
					onComplete: this.controller.stageController.deactivate.bind(this.controller.stageController)
				});
			}

			Mojo.Animation.animateStyle(this.systemMenu, 'top', 'bezier', {
				from: this.systemMenuTop,
				to: (-1 * this.systemMenu.offsetHeight),
				duration: .15,
				curve:'ease-out',
				onComplete:animateScrim.bind(this)
			});
		}
		else {
			this.isVisible = true;
			
			if(edit) {
				this.isEditing = true;

				this.controller.get('menu_items').hide();
				this.controller.get('edit_items').show();

				this.updateEditItems();			
			}
			else {			
				this.isEditing = false;

				this.eventDay = 0;

				this.controller.get('menu_items').show();
				this.controller.get('edit_items').hide();

				this.updateMenuItems();
			}
						
			this.controller.stageController.activate();
			
			var animateDialog = function(){
				Mojo.Animation.animateStyle(this.systemMenu, 'top', 'bezier', {
					from: (-1 * this.systemMenu.offsetHeight),
					to: this.systemMenuTop,
					duration: .15,
					curve:'ease-in'
				});
			}
            
			Mojo.Animation.animateStyle(this.scrim, 'opacity', 'bezier', {
				from: 0,
				to: 1,
				duration: .15,
				curve:'over-easy',
				reverse:false,
				styleSetter: function(element,value){
					element.style.opacity = value;
				}.curry(this.scrim).bind(this),
				onComplete: animateDialog.bind(this)
			});
		}
	},

	getWeek: function(date) {
		var year = date.getFullYear();
		var month = date.getMonth();
		var day = date.getDate();
		
		var when = new Date(year,month,day);
		var newYear = new Date(year,0,1);
		var modDay = newYear.getDay();

		if(this.startDay != 1)
			if (modDay == 0) modDay=6; else modDay--;

		var daynum = ((Date.UTC(year,when.getMonth(),when.getDate(),0,0,0) -
			Date.UTC(year,0,1,0,0,0)) /1000/60/60/24) + 1;

		if (modDay < 4 ) {
			var weekno = Math.floor((daynum+modDay-1)/7)+1;
		}
		else {
			var weekno = Math.floor((daynum+modDay-1)/7);
		
			if (weekno == 0) {
				year--;
			
				var prevNewYear = new Date(year,0,1);
				var prevmodDay = prevNewYear.getDay();
				if (prevmodDay == 0) prevmodDay = 6; else prevmodDay--;
				if (prevmodDay < 4) weekno = 53; else weekno = 52;
			}
		}

		return weekno;
	},
	
	sortAlldayEvents: function(a,b) {
		var c = a.title.toLowerCase();
		var d = b.title.toLowerCase();
	
		return ((c < d) ? -1 : ((c > d) ? 1 : 0));
	},
	
	sortNormalEvents: function(a,b) {
		return a.date.getTime() - b.date.getTime();
	},
});

